@@ -3,10 +3,10 @@
  *  Module    : art.c
  *  Author    : I.Lea & R.Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2015-11-11
+ *  Updated   : 2016-10-10
  *  Notes     :
  *
- * Copyright (c) 1991-2016 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2017 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -212,7 +212,7 @@ setup_hard_base(
 #ifdef NNTP_ABLE
 		char buf[NNTP_STRLEN];
 		char line[NNTP_STRLEN];
-		int getart_limit = cmdline.args & CMDLINE_GETART_LIMIT ? cmdline.getart_limit : tinrc.getart_limit;
+		int getart_limit = (cmdline.args & CMDLINE_GETART_LIMIT) ? cmdline.getart_limit : tinrc.getart_limit;
 		FILE *fp;
 		t_artnum last, start, count = 0, j = 0;
 		static t_bool skip_listgroup = FALSE;
@@ -433,7 +433,7 @@ index_group(
 	min = grpmenu.max ? base[0] : group->xmin;
 	max = grpmenu.max ? base[grpmenu.max - 1] : min - 1;
 
-	getart_limit = cmdline.args & CMDLINE_GETART_LIMIT ? cmdline.getart_limit : tinrc.getart_limit;
+	getart_limit = (cmdline.args & CMDLINE_GETART_LIMIT) ? cmdline.getart_limit : tinrc.getart_limit;
 
 	if (getart_limit > 0) {
 		if (grpmenu.max && (grpmenu.max > getart_limit))
@@ -630,6 +630,13 @@ open_art_header(
 		if ((fp = nntp_command(buf, OK_HEAD, NULL, 0)) != NULL)
 			return fp;
 
+		/*
+		 * TODO:
+		 * shall we stop on 5xx?, i.e JamNNTPd/2 1.3 responds with
+		 * "503 Access denied" instead of 480 but NEXT still works,
+		 * so tin loops over all articles without getting useful data
+		 */
+
 		/*
 		 * HEAD failed, try to find NEXT
 		 * Should return "223 artno message-id more text...."
@@ -706,7 +713,7 @@ read_art_headers(
 {
 	FILE *fp;
 	char dir[PATH_LEN];
-	char *group_msg;
+	char group_msg[LEN];
 	int i;
 	int modified = 0;
 	t_artnum art;
@@ -725,7 +732,8 @@ read_art_headers(
 		chdir(buf);
 	}
 
-	group_msg = fmt_string(_(txt_group), cCOLS - strlen(_(txt_group)) + 2 - 3, group->name);
+	snprintf(group_msg, sizeof(group_msg), _(txt_group), cCOLS - strlen(_(txt_group)) + 2 - 3, group->name);
+
 	for (i = 0; i < grpmenu.max; i++) {	/* for each article number */
 		art = base[i];
 
@@ -795,7 +803,6 @@ read_art_headers(
 		if (++modified % MODULO_COUNT_NUM == 0)
 			show_progress(group_msg, modified, total);
 	}
-	free(group_msg);
 
 	/*
 	 * Change back to previous dir before indexing started
@@ -884,7 +891,7 @@ thread_by_percentage(
 {
 	int i, j, k;
 	int root_num = 0; /* The index number of the root we are currently working on. */
-	int unmatched; /* This is the number of characters that don't match between the two strings */
+	unsigned int unmatched; /* This is the number of characters that don't match between the two strings */
 	unsigned int percentage = 100 - group->attribute->thread_perc;
 	size_t slen;
 
@@ -920,7 +927,7 @@ thread_by_percentage(
 		 */
 		if (!(slen = strlen(arts[base[root_num]].subject)))
 			slen++;
-		unmatched += abs(slen - strlen(arts[i].subject));
+		unmatched += slen - strlen(arts[i].subject);
 		if (unmatched * 100 / slen > percentage) {
 			/*
 			 * If there is less greater than percentage% different start a
@@ -1653,7 +1660,7 @@ read_overview(
 					debug_print_file("NNTP", "%s(%"T_ARTNUM_PFMT") Unexpected overview-field %d of %d: %s", nntp_caps.over_cmd, artnum, count, over_fields, ptr);
 #endif /* DEBUG */
 
-				/* "common error" Xref:full in overview-data but not in OVERVIEW.FTM */
+				/* "common error" Xref:full in overview-data but not in OVERVIEW.FMT */
 				if (count == over_fields + 1) {
 					if (!strncasecmp(ptr, "Xref: ", 6)) {
 #ifdef DEBUG
@@ -1744,7 +1751,7 @@ read_overview(
 						if (*ptr) {
 #ifdef DEBUG
 							if ((debug & DEBUG_NNTP) && !isdigit((unsigned char) *ptr))
-									debug_print_file("NNTP", "%s(%"T_ARTNUM_PFMT") overview field %d (%s) mismatch: %s", nntp_caps.over_cmd, artnum, count, ofmt[count].name, ptr);
+								debug_print_file("NNTP", "%s(%"T_ARTNUM_PFMT") overview field %d (%s) mismatch: %s", nntp_caps.over_cmd, artnum, count, ofmt[count].name, ptr);
 #endif /* DEBUG */
 						}
 						continue;
@@ -2018,7 +2025,7 @@ write_overview(
 	/*
 	 * Can't write or caching is off or getart_limit is set
 	 */
-	if (no_write || !tinrc.cache_overview_files || (cmdline.args & CMDLINE_GETART_LIMIT ? cmdline.getart_limit : tinrc.getart_limit) != 0)
+	if (no_write || !tinrc.cache_overview_files || ((cmdline.args & CMDLINE_GETART_LIMIT) ? cmdline.getart_limit : tinrc.getart_limit) != 0)
 		return;
 
 	if ((fp = open_xover_fp(group, "w", T_ARTNUM_CONST(0), T_ARTNUM_CONST(0), FALSE)) == NULL)
@@ -2210,15 +2217,18 @@ find_nov_file(
 			 * Append -<nntpserver> to private cache dir
 			 */
 			if (!once_only && nntp_server) {
-				const char *from;
-				char *to;
-				int c;
+				size_t sp, ln = strlen(index_newsdir);
 
-				to = index_newsdir + strlen(index_newsdir);
-				*(to++) = '-';
-				for (from = nntp_server; (c = *from) != 0; ++from)
-					*(to++) = tolower(c);
-				*to = '\0';
+				if ((sp = sizeof(index_newsdir) - ln - 1) >= 2) {
+					char *srv = my_strdup(nntp_server);
+
+					strcat(index_newsdir, "-");
+					sp--;
+					ln++;
+					str_lwr(srv);
+					my_strncpy(index_newsdir + ln, srv, sp);
+					free(srv);
+				}
 				once_only = TRUE;
 			}
 
@@ -2284,7 +2294,6 @@ find_nov_file(
 
 		if (strcmp(buf, group->name) == 0)
 			break;
-
 	}
 
 	return nov_file;
@@ -2753,7 +2762,11 @@ find_artnum(
 
 
 /*----------------------------- Overview handling -----------------------*/
-
+/* TODO: use
+ *           setlocale(LC_ALL, "POSIX"); setlocale(LC_TIME, "POSIX");
+ *           my_strftime(date, sizeof(date) -1, "%d %b %Y %H:%M:%S GMT", gmtime(&secs));
+ *       instead?
+ */
 static char *
 print_date(
 	time_t secs)
@@ -2765,12 +2778,14 @@ print_date(
 		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
 	};
 
-	tm = gmtime(&secs);
-	snprintf(date, sizeof(date), "%02d %s %04d %02d:%02d:%02d GMT",
-			tm->tm_mday,
-			months_a[tm->tm_mon],
-			tm->tm_year + 1900,
-			tm->tm_hour, tm->tm_min, tm->tm_sec);
+	if ((tm = gmtime(&secs)) != NULL)
+		snprintf(date, sizeof(date), "%02d %.3s %04d %02d:%02d:%02d GMT",
+				tm->tm_mday,
+				months_a[tm->tm_mon],
+				tm->tm_year + 1900,
+				tm->tm_hour, tm->tm_min, tm->tm_sec);
+	else
+		snprintf(date, sizeof(date), "01 Jan 1970 00:00:00 UTC");
 
 	return date;
 }
