@@ -3,10 +3,10 @@
  *  Module    : config.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2015-11-22
+ *  Updated   : 2016-10-10
  *  Notes     : Configuration file routines
  *
- * Copyright (c) 1991-2016 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2017 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -694,8 +694,10 @@ read_config_file(
 			if (match_string(buf, "strokes_regex=", tinrc.strokes_regex, sizeof(tinrc.strokes_regex)))
 				break;
 
+#ifndef USE_CURSES
 			if (match_boolean(buf, "strip_blanks=", &tinrc.strip_blanks))
 				break;
+#endif /* !USE_CURSES */
 
 			if (match_integer(buf, "strip_bogus=", &tinrc.strip_bogus, BOGUS_SHOW))
 				break;
@@ -764,7 +766,7 @@ read_config_file(
 
 #ifdef HAVE_COLOR
 			if (match_boolean(buf, "use_color=", &tinrc.use_color)) {
-				use_color = cmdline.args & CMDLINE_USE_COLOR ? bool_not(tinrc.use_color) : tinrc.use_color;
+				use_color = (cmdline.args & CMDLINE_USE_COLOR) ? bool_not(tinrc.use_color) : tinrc.use_color;
 				break;
 			}
 #endif /* HAVE_COLOR */
@@ -1181,8 +1183,10 @@ write_config_file(
 	fprintf(fp, "%s", _(txt_use_mouse.tinrc));
 	fprintf(fp, "use_mouse=%s\n\n", print_boolean(tinrc.use_mouse));
 
+#ifndef USE_CURSES
 	fprintf(fp, "%s", _(txt_strip_blanks.tinrc));
 	fprintf(fp, "strip_blanks=%s\n\n", print_boolean(tinrc.strip_blanks));
+#endif /* !USE_CURSES */
 
 	fprintf(fp, "%s", _(txt_abbreviate_groupname.tinrc));
 	fprintf(fp, "abbreviate_groupname=%s\n\n", print_boolean(tinrc.abbreviate_groupname));
