@@ -3,10 +3,10 @@
  *  Module    : init.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2015-11-21
+ *  Updated   : 2016-04-17
  *  Notes     :
  *
- * Copyright (c) 1991-2016 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2017 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -385,7 +385,9 @@ struct t_config tinrc = {
 #else
 	FALSE,		/* start_editor_offset */
 #endif /* M_UNIX */
+#ifndef USE_CURSES
 	TRUE,		/* strip_blanks */
+#endif /* !USE_CURSES */
 	FALSE,		/* strip_newsrc */
 	FALSE,		/* tex2iso_conv */
 	TRUE,		/* thread_catchup_on_exit */
