@@ -3,10 +3,10 @@
  *  Module    : page.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2015-12-16
+ *  Updated   : 2016-07-29
  *  Notes     :
  *
- * Copyright (c) 1991-2016 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2017 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,14 +42,6 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 
-#if 0
-#if defined(HAVE_IDNA_H) && !defined(_IDNA_H)
-#	include <idna.h>
-#endif /* HAVE_IDNA_H && !_IDNA_H */
-#if defined(HAVE_STRINGPREP_H) && !defined(_STRINGPREP_H)
-#	include <stringprep.h>
-#endif /* HAVE_STRINGPREP_H && !_STRINGPREP_H */
-#endif /* 0 */
 
 /*
  * PAGE_HEADER is the size in lines of the article page header
@@ -1523,7 +1515,12 @@ draw_page_header(
 		free(wtmp);
 	}
 
-	/* organization */
+	/*
+	 * Organization
+	 *
+	 * TODO: IDNA decoding, see also comment in
+	 *       cook.c:cook_article()
+	 */
 	if ((wtmp = char2wchar_t(_(txt_at_s))) != NULL) {
 		len = wcswidth(wtmp, wcslen(wtmp));
 		free(wtmp);
@@ -1761,13 +1758,10 @@ load_article(
 #endif /* DEBUG */
 
 	if (new_respnum != this_resp || art_closed) {
-		char *progress_mesg = my_strdup(_(txt_reading_article));
 		int ret;
 
 		art_close(&pgart);			/* close previously opened art in pager */
-
-		ret = art_open(TRUE, &arts[new_respnum], group, &pgart, TRUE, progress_mesg);
-		free(progress_mesg);
+		ret = art_open(TRUE, &arts[new_respnum], group, &pgart, TRUE, _(txt_reading_article));
 
 		switch (ret) {
 			case ART_UNAVAILABLE:
@@ -2075,10 +2069,8 @@ resize_article(
 	if (artinfo->cooked)
 		fclose(artinfo->cooked);
 
-	if (!cook_article(wrap_lines, artinfo, hide_uue, show_all_headers)) {
-		wait_message(3, _(txt_cook_article_failed_exiting), tin_progname);
-		tin_done(EXIT_FAILURE);
-	}
+	if (!cook_article(wrap_lines, artinfo, hide_uue, show_all_headers))
+		tin_done(EXIT_FAILURE, _(txt_cook_article_failed_exiting), tin_progname);
 
 	show_raw_article = FALSE;
 	artline = pgart.cookl;
