@@ -3,10 +3,10 @@
  *  Module    : rfc2047.c
  *  Author    : Chris Blum <chris@resolution.de>
  *  Created   : 1995-09-01
- *  Updated   : 2015-12-18
+ *  Updated   : 2016-03-10
  *  Notes     : MIME header encoding/decoding stuff
  *
- * Copyright (c) 1995-2016 Chris Blum <chris@resolution.de>
+ * Copyright (c) 1995-2017 Chris Blum <chris@resolution.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -162,8 +162,10 @@ mmdecode(
 			if (*what != '=') {
 				if (!delimiter || *what != '_')
 					*t++ = *what++;
-				else
-					*t++ = ' ', what++;
+				else {
+					*t++ = ' ';
+					what++;
+				}
 				continue;
 			}
 			what++;
@@ -301,8 +303,8 @@ rfc1522_decode(
 
 						i = mmdecode(c, encoding, '?', t);
 						if (i > 0) {
-							int chars_to_copy;
 							char *tmpbuf;
+							int chars_to_copy;
 
 							max_len = i + 1;
 							tmpbuf = my_malloc(max_len);
