@@ -3,10 +3,10 @@
  *  Module    : save.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2015-12-16
+ *  Updated   : 2016-10-10
  *  Notes     :
  *
- * Copyright (c) 1991-2016 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2017 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -49,16 +49,21 @@
 #endif /* HAVE_UUDEVIEW_H */
 
 #ifndef HAVE_LIBUU
-#undef OFF
-enum state { INITIAL, MIDDLE, OFF, END };
+#	undef OFF
+	enum state {
+		INITIAL,
+		MIDDLE,
+		OFF,
+		END
+	};
 #endif /* !HAVE_LIBUU */
 
 enum action {
 	VIEW,
 	SAVE,
-	SAVE_TAGGED
+	SAVE_TAGGED,
+	PIPE_RAW
 #ifndef DONT_HAVE_PIPING
-	, PIPE_RAW
 	, PIPE
 #endif /* !DONT_HAVE_PIPING */
 };
@@ -208,7 +213,7 @@ check_start_save_any_news(
 			if (function == SAVE_ANY_NEWS) {
 				char tmp[PATH_LEN];
 
-				if (!strfpath(cmdline.args & CMDLINE_SAVEDIR ? cmdline.savedir : tinrc.savedir, tmp, sizeof(tmp), group, FALSE))
+				if (!strfpath((cmdline.args & CMDLINE_SAVEDIR) ? cmdline.savedir : tinrc.savedir, tmp, sizeof(tmp), group, FALSE))
 					joinpath(tmp, sizeof(tmp), homedir, DEFAULT_SAVEDIR);
 
 				make_group_path(group->name, group_path);
@@ -596,26 +601,34 @@ t_bool
 create_path(
 	const char *path)
 {
-	char buf[PATH_LEN];
-	int i, j, len;
+	char *buf, *p;
 	struct stat st;
 
-	len = (int) strlen(path);
+	if (!strlen(path))
+		return FALSE;
 
-	for (i = 0, j = 0; i < len; i++, j++) {
-		buf[j] = path[i];
-		if (i + 1 < len && path[i + 1] == DIRSEP) {
-			buf[j + 1] = '\0';
-			if (stat(buf, &st) == -1) {
-				if (my_mkdir(buf, (mode_t) (S_IRWXU|S_IRUGO|S_IXUGO)) == -1) {
-					if (errno != EEXIST) {
-						perror_message(_(txt_cannot_create), buf);
-						return FALSE;
-					}
+	buf = my_strdup(path);
+	p = buf + 1;
+
+	if (!strlen(p)) {
+		free(buf);
+		return FALSE;
+	}
+
+	while ((p = strchr(p, DIRSEP)) != NULL) {
+		*p = '\0';
+		if (stat(buf, &st) == -1) {
+			if (my_mkdir(buf, (mode_t) (S_IRWXU|S_IRUGO|S_IXUGO)) == -1) {
+				if (errno != EEXIST) {
+					perror_message(_(txt_cannot_create), buf);
+					free(buf);
+					return FALSE;
 				}
 			}
 		}
+		*p++ = DIRSEP;
 	}
+	free(buf);
 	return TRUE;
 }
 
@@ -635,6 +648,7 @@ generate_filename(
 	snprintf(buf, buflen, "%s-%03d.%s", SAVEFILE_PREFIX, seqno++, suffix);
 }
 
+
 /*
  * Generate /save/to/path name.
  *
@@ -707,7 +721,7 @@ expand_save_filename(
 	base_name(buf_path, base_filename);
 
 	/* Build default path to save to */
-	if (!(ret = strfpath(cmdline.args & CMDLINE_SAVEDIR ? cmdline.savedir : curr_group->attribute->savedir, buf, sizeof(buf), curr_group, FALSE)))
+	if (!(ret = strfpath((cmdline.args & CMDLINE_SAVEDIR) ? cmdline.savedir : curr_group->attribute->savedir, buf, sizeof(buf), curr_group, FALSE)))
 		joinpath(buf, sizeof(buf), homedir, DEFAULT_SAVEDIR);
 
 	/* Join path and filename */
@@ -936,6 +950,7 @@ post_process_uud(
 						filename = strrchr(path, DIRSEP) + 1;	/* ptr to filename portion */
 						if ((fp_out = fopen(path, "w")) == NULL) {
 							perror_message(_(txt_cannot_open), path);
+							fclose(fp_in);
 							return;
 						}
 						state = MIDDLE;
@@ -1082,10 +1097,9 @@ view_file(
 	/*
 	 * Needed for the mime-type processor
 	 */
-	part->params = my_malloc(sizeof(t_param));
+	part->params = new_params();
 	part->params->name = my_strdup("name");
 	part->params->value = my_strdup(file);
-	part->params->next = NULL;
 
 	start_viewer(part, path);
 	my_printf(cCRLF);
@@ -1363,7 +1377,11 @@ decode_save_one(
 }
 
 
-enum match { NO, MATCH, NOTMATCH };
+enum match {
+	NO,
+	MATCH,
+	NOTMATCH
+};
 
 /*
  * Match a single type/subtype Content pair
@@ -1831,7 +1849,11 @@ build_attachment_line(
 
 	charset = get_param(part->params, "charset");
 	snprintf(buf2, sizeof(buf2), _(txt_attachment_lines), part->line_count);
-	snprintf(buf, sizeof(buf), "  %s/%s, %s, %s%s%s", content_types[part->type], part->subtype, content_encodings[part->encoding], charset ? charset : "", charset ? ", " : "", buf2);
+	/* TODO: make the layout configurable? */
+	if (!strcmp(content_types[part->type], "text"))
+		snprintf(buf, sizeof(buf), "  %s/%s, %s, %s%s%s", content_types[part->type], part->subtype, content_encodings[part->encoding], charset ? charset : "", charset ? ", " : "", buf2);
+	else
+		snprintf(buf, sizeof(buf), "  %s/%s, %s, %s", content_types[part->type], part->subtype, content_encodings[part->encoding], buf2);
 	if (part->depth > 0) {
 		treelen = cCOLS - 13 - info_len - namelen;
 		tree = build_tree(part->depth, treelen, i);
@@ -2034,6 +2056,7 @@ tag_pattern(
 	return;
 }
 
+
 static int
 get_tagged(
 	int n)
@@ -2321,8 +2344,8 @@ process_part(
 				default:
 #ifdef CHARSET_CONVERSION
 						if (what != SAVE && what != SAVE_TAGGED && !strncmp(content_types[part->type], "text", 4)) {
-							line_len = strlen(buf);
 							conv_buf = my_strdup(buf);
+							line_len = strlen(conv_buf);
 							network_charset = get_param(part->params, "charset");
 							process_charsets(&conv_buf, &line_len, network_charset ? network_charset : "US-ASCII", tinrc.mm_local_charset, FALSE);
 							strncpy(buf, conv_buf, sizeof(buf) - 1);
