@@ -3,10 +3,10 @@
  *  Module    : string.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-01-20
- *  Updated   : 2015-11-28
+ *  Updated   : 2016-02-25
  *  Notes     :
  *
- * Copyright (c) 1997-2016 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2017 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -250,10 +250,9 @@ str_lwr(
 {
 	char *dst = str;
 
-	while (*str) {
-		*dst++ = (char) tolower((unsigned char) *str);
-		str++;
-	}
+	while (*str)
+		*dst++ = (char) tolower((unsigned char) *str++);
+
 	*dst = '\0';
 }
 
@@ -664,22 +663,22 @@ sh_format(
 			}
 
 			switch (*fmt++) {
-			case '%':
-				src = strcpy(temp, "%");
-				break;
+				case '%':
+					src = strcpy(temp, "%");
+					break;
 
-			case 's':
-				src = va_arg(ap, char *);
-				break;
+				case 's':
+					src = va_arg(ap, char *);
+					break;
 
-			case 'd':
-				snprintf(temp, sizeof(temp), "%d", va_arg(ap, int));
-				src = temp;
-				break;
+				case 'd':
+					snprintf(temp, sizeof(temp), "%d", va_arg(ap, int));
+					src = temp;
+					break;
 
-			default:
-				src = strcpy(temp, "");
-				break;
+				default:
+					src = strcpy(temp, "");
+					break;
 			}
 
 			while (*src != '\0') {
@@ -1274,6 +1273,7 @@ normalize(
 				mode = UNINORM_NFKC;
 		}
 		buf = (char *) u8_normalize(mode, (uint8_t *) tmp, strlen(tmp) + 1, NULL, &olen);
+		free(tmp);
 		return buf;
 	}
 #		else
