@@ -3,10 +3,10 @@
  *  Module    : thread.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2015-11-09
+ *  Updated   : 2016-07-26
  *  Notes     :
  *
- * Copyright (c) 1991-2016 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2017 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -146,7 +146,7 @@ build_tline(
 	buffer[0] = '\0';
 
 	if (tinrc.draw_arrow)
-			strcat(buffer, "  ");
+		strcat(buffer, "  ");
 
 	for (; *fmt; fmt++) {
 		if (*fmt != '%') {
@@ -193,13 +193,15 @@ build_tline(
 					free(wtmp2);
 				}
 #else
-				len_start = strwidth(buffer);
-				get_author(TRUE, art, buffer + strlen(buffer), thrd_fmt.len_from);
-				fill = thrd_fmt.len_from - (strwidth(buffer) - len_start);
-				gap = strlen(buffer);
-				for (i = 0; i < fill; i++)
-					buffer[gap + i] = ' ';
-				buffer[gap + fill] = '\0';
+				if (curr_group->attribute->show_author != SHOW_FROM_NONE) {
+					len_start = strwidth(buffer);
+					get_author(TRUE, art, buffer + strlen(buffer), thrd_fmt.len_from);
+					fill = thrd_fmt.len_from - (strwidth(buffer) - len_start);
+					gap = strlen(buffer);
+					for (i = 0; i < fill; i++)
+						buffer[gap + i] = ' ';
+					buffer[gap + fill] = '\0';
+				}
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 				break;
 
@@ -342,14 +344,14 @@ build_tline(
 
 					default:
 						break;
-					}
+				}
 
-					/* pad out */
-					fill = len - (strwidth(buffer) - len_start);
-					gap = strlen(buffer);
-					for (i = 0; i < fill; i++)
-						buffer[gap + i] = ' ';
-					buffer[gap + fill] = '\0';
+				/* pad out */
+				fill = len - (strwidth(buffer) - len_start);
+				gap = strlen(buffer);
+				for (i = 0; i < fill; i++)
+					buffer[gap + i] = ' ';
+				buffer[gap + fill] = '\0';
 				break;
 
 			default:
@@ -359,15 +361,10 @@ build_tline(
 	/* protect display from non-displayable characters (e.g., form-feed) */
 	convert_to_printable(buffer, FALSE);
 
-	if (!tinrc.strip_blanks) {
-		/* Pad to end of line so that inverse bar looks 'good' */
-		fill = cCOLS - strwidth(buffer);
-		gap = strlen(buffer);
-		for (i = 0; i < fill; i++)
-			buffer[gap + i] = ' ';
-
-		buffer[gap + fill] = '\0';
-	}
+#ifndef USE_CURSES
+	if (tinrc.strip_blanks)
+		strcat(strip_line(buffer), cCRLF);
+#endif /* !USE_CURSES */
 
 	WriteLine(INDEX2LNUM(l), buffer);
 
@@ -624,12 +621,10 @@ thread_page(
 
 			case THREAD_CANCEL:		/* cancel current article */
 				if (can_post || group->attribute->mailing_list != NULL) {
-					char *progress_msg = my_strdup(_(txt_reading_article));
 					int ret;
 
 					n = find_response(thread_basenote, thdmenu.curr);
-					ret = art_open(TRUE, &arts[n], group, &pgart, TRUE, progress_msg);
-					free(progress_msg);
+					ret = art_open(TRUE, &arts[n], group, &pgart, TRUE, _(txt_reading_article));
 					if (ret != ART_UNAVAILABLE && ret != ART_ABORT && cancel_article(group, &arts[n], n))
 						show_thread_page();
 					art_close(&pgart);
@@ -1124,7 +1119,7 @@ stat_thread(
 	struct t_art_stat *sbuf) /* return value is always ignored */
 {
 	int i;
-	MultiPartInfo minfo;
+	MultiPartInfo minfo = { 0 };
 
 	sbuf->total = 0;
 	sbuf->unread = 0;
