@@ -4,7 +4,7 @@
 # signs the article and posts it.
 #
 #
-# Copyright (c) 2002-2016 Urs Janssen <urs@tin.org>,
+# Copyright (c) 2002-2017 Urs Janssen <urs@tin.org>,
 #                         Marc Brockschmidt <marc@marcbrockschmidt.de>
 #
 # Redistribution and use in source and binary forms, with or without
@@ -39,14 +39,15 @@
 #       - add $PGPOPTS, $PGPPATH and $GNUPGHOME support
 #       - cleanup and remove duplicated code
 #       - option to convert CRLF to LF in input
-#       - use STARTTLS (if Net::NNTP is recent enought and server supports it)
+#       - use STARTTLS (if Net::NNTP is recent enough and server supports it)
+#       - quote inpupt properly before passing to shell
 #
 
 use strict;
 use warnings;
 
 # version Number
-my $version = "1.1.40";
+my $version = "1.1.42";
 
 my %config;
 
@@ -59,10 +60,10 @@ $config{'NNTPPass'}		= '';	# password for nntp-auth, may be set via ~/.newsauth
 $config{'PGPSigner'}	= '';	# sign as who?
 $config{'PGPPass'}		= '';	# pgp2 only
 $config{'PathtoPGPPass'}= '';	# pgp2, pgp5, pgp6 and gpg
-$config{'PGPPassFD'}	= 9;	# file descriptor used for input redirection of PathtoPGPPass, GPG, PGP5 and PGP6 only
+$config{'PGPPassFD'}	= 9;	# file descriptor used for input redirection of PathtoPGPPass; GPG1, GPG2, PGP5 and PGP6 only
 
 $config{'pgp'}			= '/usr/bin/pgp';	# path to pgp
-$config{'PGPVersion'}	= '2';	# Use 2 for 2.X, 5 for PGP5, 6 for PGP6 and GPG for GPG
+$config{'PGPVersion'}	= '2';	# Use 2 for 2.X, 5 for PGP5, 6 for PGP6, GPG or GPG1 for GPG1 and GPG2 for GPG2
 $config{'digest-algo'}	= 'MD5';# Digest Algorithm for GPG. Must be supported by your installation
 
 $config{'Interactive'}	= 'yes';# allow interactive usage
@@ -81,6 +82,8 @@ $config{'pgpend'}		= '-----END PGP SIGNATURE-----';	# End of PGP-Signature
 
 # $config{'canlock_secret'}	= '~/.cancelsecret';		# Path to canlock secret file
 
+# $config{'ignore_headers'} = '';      # headers to be ignored during signing
+
 $config{'PGPSignHeaders'} = ['From', 'Newsgroups', 'Subject', 'Control',
 	'Supersedes', 'Followup-To', 'Date', 'Injection-Date', 'Sender', 'Approved',
 	'Message-ID', 'Reply-To', 'Cancel-Key', 'Also-Control',
@@ -148,6 +151,7 @@ GetOptions('A|V|W|O|no-organization|h|headers' => [], # do nothing
 	'distribution|d=s'	=> \$config{'distribution'},
 	'expires|e=s'	=> \$config{'expires'},
 	'from|f=s'	=> \$config{'from'},
+	'ignore-headers|i=s'	=> \$config{'ignore_headers'},
 	'followupto|w=s'	=> \$config{'followup-to'},
 	'newsgroups|n=s'	=> \$config{'newsgroups'},
 	'replyto|r=s'	=> \$config{'reply-to'},
@@ -211,7 +215,13 @@ if (! $config{'no_sign'}) {
 	}
 }
 
-
+# Remove unwanted headers from PGPSignHeaders
+if (${config{'ignore_headers'}}) {
+	my @hdr_to_ignore = split(/,/, ${config{'ignore_headers'}});
+	foreach my $hdr (@hdr_to_ignore) {
+		@{$config{'PGPSignHeaders'}} = map {lc($_) eq lc($hdr) ? () : $_} @{$config{'PGPSignHeaders'}};
+	}
+}
 # Read the message and split the header
 readarticle(\%Header, \@Body);
 
@@ -621,11 +631,19 @@ sub getpgpcommand {
 		} else {
 			die("$0: Passphrase is unknown!\n");
 		}
-	} elsif ($PGPVersion =~ m/GPG/io) {
+	} elsif ($PGPVersion =~ m/GPG1?$/io) {
 		if ($config{'PathtoPGPPass'}) {
-			$PGPCommand = $config{'pgp'}." --digest-algo $config{'digest-algo'} -a -u \"".$config{'PGPSigner'}."\" -o ".$config{'pgptmpf'}.".txt.asc --no-tty --batch --passphrase-fd ".$config{'PGPPassFD'}." ".$config{'PGPPassFD'}."<".$config{'PathtoPGPPass'}." --clearsign ".$config{'pgptmpf'}.".txt";
+			$PGPCommand = $config{'pgp'}." --emit-version --digest-algo $config{'digest-algo'} -a -u \"".$config{'PGPSigner'}."\" -o ".$config{'pgptmpf'}.".txt.asc --no-tty --batch --passphrase-fd ".$config{'PGPPassFD'}." ".$config{'PGPPassFD'}."<".$config{'PathtoPGPPass'}." --clearsign ".$config{'pgptmpf'}.".txt";
 		} elsif ($config{'Interactive'}) {
-			$PGPCommand = $config{'pgp'}." --digest-algo $config{'digest-algo'} -a -u \"".$config{'PGPSigner'}."\" -o ".$config{'pgptmpf'}.".txt.asc --no-secmem-warning --no-batch --clearsign ".$config{'pgptmpf'}.".txt";
+			$PGPCommand = $config{'pgp'}." --emit-version --digest-algo $config{'digest-algo'} -a -u \"".$config{'PGPSigner'}."\" -o ".$config{'pgptmpf'}.".txt.asc --no-secmem-warning --no-batch --clearsign ".$config{'pgptmpf'}.".txt";
+		} else {
+			die("$0: Passphrase is unknown!\n");
+		}
+	} elsif ($PGPVersion =~ m/GPG2$/io) {
+		if ($config{'PathtoPGPPass'}) {
+			$PGPCommand = $config{'pgp'}." --pinentry-mode loopback --emit-version --digest-algo $config{'digest-algo'} -a -u \"".$config{'PGPSigner'}."\" -o ".$config{'pgptmpf'}.".txt.asc --no-tty --batch --passphrase-fd ".$config{'PGPPassFD'}." ".$config{'PGPPassFD'}."<".$config{'PathtoPGPPass'}." --clearsign ".$config{'pgptmpf'}.".txt";
+		} elsif ($config{'Interactive'}) {
+			$PGPCommand = $config{'pgp'}." --emit-version --digest-algo $config{'digest-algo'} -a -u \"".$config{'PGPSigner'}."\" -o ".$config{'pgptmpf'}.".txt.asc --no-secmem-warning --no-batch --clearsign ".$config{'pgptmpf'}.".txt";
 		} else {
 			die("$0: Passphrase is unknown!\n");
 		}
@@ -803,6 +821,7 @@ sub usage {
 	print "  -d string  set Distribution:-header to string\n";
 	print "  -e string  set Expires:-header to string\n";
 	print "  -f string  set From:-header to string\n";
+	print "  -i string  list of headers to be ignored for signing\n";
 	print "  -n string  set Newsgroups:-header to string\n";
 	print "  -o string  set Organization:-header to string\n";
 	print "  -p port    use port as NNTP port [default=".$config{'NNTPPort'}."]\n";
@@ -876,6 +895,20 @@ X<-f> X<--from>
 
 Set the article header field From: to the given value.
 
+=item -B<i> F<header> | --B<ignore-headers> F<header>
+X<-i> X<--ignore-headers>
+
+Comma separated list of headers that will be ignored during signing.
+Usually the following headers will be signed if present:
+
+From, Newsgroups, Subject, Control, Supersedes, Followup-To,
+Date, Injection-Date, Sender, Approved, Message-ID, Reply-To,
+Cancel-Key, Also-Control and Distribution.
+
+Some of them may be altered on the Server (i.e. Cancel-Key) which would
+invalid the signature, this option can be used the exclude such headers
+if required.
+
 =item -B<n> C<Newsgroups> | --B<newsgroups> C<Newsgroups>
 X<-n> X<--newsgroups>
 
@@ -1082,7 +1115,7 @@ off by default.
 "nntpserver password [user]" pairs for NNTP servers that require
 authorization. Any line that starts with "#" is a comment. Blank lines are
 ignored. This file should be readable only for the user as it contains the
-user's uncrypted password for reading news. First match counts. If no
+user's unencrypted password for reading news. First match counts. If no
 matching entry is found F<$HOME/.nntpauth> is checked.
 
 =item F<$HOME/.nntpauth>
@@ -1090,13 +1123,13 @@ matching entry is found F<$HOME/.nntpauth> is checked.
 "nntpserver user password" pairs for NNTP servers that require
 authorization. First match counts. Lines starting with "#" are skipped and
 blank lines are ignored. This file should be readable only for the user as
-it contains the user's uncrypted password for reading news.
+it contains the user's unencrypted password for reading news.
 F<$HOME/.newsauth> is checked first.
 
 =item F<$XDG_CONFIG_HOME/tinewsrc> F<$HOME/.config/tinewsrc> F<$HOME/.tinewsrc>
 
 "option=value" configuration pairs. Lines that start with "#" are ignored.
-If the file contains uncrypted passwords (e.g. NNTPPass or PGPPass), it
+If the file contains unencrypted passwords (e.g. NNTPPass or PGPPass), it
 should be readable for the user only.
 
 =back
@@ -1111,7 +1144,7 @@ security is an issue, don't use this script.
 =head1 NOTES
 
 B<tinews.pl> is designed to be used with B<pgp>(1)-2.6.3,
-B<pgp>(1)-5, B<pgp>(1)-6 and B<gpg>(1).
+B<pgp>(1)-5, B<pgp>(1)-6, B<gpg>(1) and B<gpg2>(1).
 
 B<tinews.pl> requires the following standard modules to be installed:
 B<Getopt::Long>(3pm), B<Net::NNTP>(3pm), B<Time::Local>(3pm) and
@@ -1128,7 +1161,7 @@ Marc Brockschmidt E<lt>marc@marcbrockschmidt.deE<gt>
 
 =head1 SEE ALSO
 
-B<pgp>(1), B<gpg>(1), B<pgps>(1), B<Digest::HMAC_SHA1>(3pm),
+B<pgp>(1), B<gpg>(1), B<gpg2>(1), B<pgps>(1), B<Digest::HMAC_SHA1>(3pm),
 B<Digest::SHA>(3pm), B<Digest::SHA1>(3pm), B<Getopt::Long>(3pm),
 B<MIME::Base64>(3pm), B<Net::NNTP>(3pm), B<Time::Local>(3pm),
 B<Term::Readline>(3pm)
